@@ -14,7 +14,7 @@ from debusine.artifacts.playground import ArtifactPlayground
 from debusine.test import TestCase
 
 
-class ArtifactPlaygroundTest(TestCase):
+class ArtifactPlaygroundTests(TestCase):
     """Test artifact playground behaviour."""
 
     def test_source_package_defaults(self) -> None:
@@ -135,10 +135,7 @@ class ArtifactPlaygroundTest(TestCase):
             "Changes": "\n".join(
                 (
                     "",
-                    (
-                        f"  hello ({expected_version}) unstable; "
-                        f"urgency=medium"
-                    ),
+                    f"  hello ({expected_version}) unstable; urgency=medium",
                     "  .",
                     "    * Test upload.",
                 )
