@@ -15,18 +15,20 @@ import json
 import re
 import stat
 from collections.abc import Generator
+from functools import partial
 from pathlib import Path
 from typing import Any
 from unittest import mock
 from unittest.mock import MagicMock
-from urllib.parse import quote, urljoin
 
 import responses
 import yaml
 
-from debusine.artifacts.models import ArtifactCategory
+from debusine.artifacts.models import ArtifactCategory, WorkRequestResults
 from debusine.client.cli import Cli
+from debusine.client.commands.base import Command
 from debusine.client.commands.tests.base import BaseCliTests
+from debusine.client.commands.work_requests import WorkRequestModelCommand
 from debusine.client.debusine import Debusine
 from debusine.client.exceptions import DebusineError
 from debusine.client.models import (
@@ -34,7 +36,6 @@ from debusine.client.models import (
     WorkRequestExternalDebsignRequest,
     WorkRequestRequest,
     WorkRequestResponse,
-    model_to_json_serializable_dict,
 )
 from debusine.test.test_utils import (
     create_artifact_response,
@@ -45,6 +46,232 @@ from debusine.test.test_utils import (
 from debusine.utils import calculate_hash
 
 
+class WorkRequestModelCommandTests(BaseCliTests):
+    """Tests for :py:class:`WorkRequestModelCommand`."""
+
+    @Command.preserve_registry()
+    def _command(self) -> WorkRequestModelCommand:
+        class ConcreteWorkRequestModelCommand(WorkRequestModelCommand):
+            """Concrete version of WorkRequestModelCommand."""
+
+            def run(self) -> None:
+                raise NotImplementedError()
+
+        return self.command(
+            ConcreteWorkRequestModelCommand, workspace="workspace"
+        )
+
+    def test_list_rich(self) -> None:
+        wr = create_work_request_response()
+        command = self._command()
+        webui_url = command.debusine.webui_url(wr.get_link_url("webui_self"))
+        with (
+            mock.patch(
+                "debusine.client.commands.work_requests.Table",
+                return_value=(table := mock.MagicMock()),
+            ),
+            mock.patch(
+                "debusine.client.commands.work_requests.rich.print"
+            ) as rich_print,
+            mock.patch("rich.json.JSON.from_data", side_effect=lambda x: x),
+        ):
+            stderr, stdout = self.capture_output(
+                partial(command._list_rich, [wr])
+            )
+
+        self.assertEqual(stderr, "")
+        self.assertEqual(stdout, "")
+        rich_print.assert_called_once()
+
+        self.assertEqual(table.add_row.call_count, 1)
+        call = table.add_row.call_args_list[0]
+        self.assertEqual(call.args[0], f"[link={webui_url}]#{wr.id}[/]")
+        self.assertEqual(call.args[1], f"{wr.task_type} {wr.task_name}")
+        self.assertEqual(call.args[2], wr.status)
+        self.assertEqual(
+            call.args[3], f"Created: {command._format_timestamp(wr.created_at)}"
+        )
+        self.assertEqual(call.args[4], command._format_int(wr.worker))
+        self.assertEqual(call.args[5], wr.task_data)
+
+    def test_list_rich_full_data(self) -> None:
+        wr = create_work_request_response()
+        wr.started_at = wr.created_at + dt.timedelta(minutes=2)
+        wr.completed_at = wr.started_at + dt.timedelta(minutes=3)
+        wr.result = WorkRequestResults.SUCCESS
+        command = self._command()
+        webui_url = command.debusine.webui_url(wr.get_link_url("webui_self"))
+        with (
+            mock.patch(
+                "debusine.client.commands.work_requests.Table",
+                return_value=(table := mock.MagicMock()),
+            ),
+            mock.patch(
+                "debusine.client.commands.work_requests.rich.print"
+            ) as rich_print,
+            mock.patch("rich.json.JSON.from_data", side_effect=lambda x: x),
+        ):
+            stderr, stdout = self.capture_output(
+                partial(command._list_rich, [wr])
+            )
+
+        self.assertEqual(stderr, "")
+        self.assertEqual(stdout, "")
+        rich_print.assert_called_once()
+
+        self.assertEqual(table.add_row.call_count, 1)
+        call = table.add_row.call_args_list[0]
+        self.assertEqual(call.args[0], f"[link={webui_url}]#{wr.id}[/]")
+        self.assertEqual(call.args[1], f"{wr.task_type} {wr.task_name}")
+        self.assertEqual(call.args[2], f"{wr.status} {wr.result}")
+        self.assertEqual(
+            call.args[3],
+            "\n".join(
+                [
+                    f"Created: {command._format_timestamp(wr.created_at)}",
+                    f"Started: {command._format_timestamp(wr.started_at)}",
+                    f"Completed: {command._format_timestamp(wr.completed_at)}",
+                ]
+            ),
+        )
+        self.assertEqual(call.args[4], command._format_int(wr.worker))
+        self.assertEqual(call.args[5], wr.task_data)
+
+    def test_format_timestamp(self) -> None:
+        command = self._command()
+        for value, expected in (
+            (None, None),
+            (dt.datetime(2026, 1, 1, tzinfo=dt.UTC), "2026-01-01 00:00:00"),
+            (
+                dt.datetime(2026, 1, 1, microsecond=42, tzinfo=dt.UTC),
+                "2026-01-01 00:00:00",
+            ),
+        ):
+            with self.subTest(value=repr(value)):
+                self.assertEqual(command._format_timestamp(value), expected)
+
+    def test_format_int(self) -> None:
+        command = self._command()
+        for value, expected in ((None, None), (0, "0"), (42, "42")):
+            with self.subTest(value=value):
+                self.assertEqual(command._format_int(value), expected)
+
+    def test_show_rich(self) -> None:
+        wr = create_work_request_response()
+        command = self._command()
+        webui_url = command.debusine.webui_url(wr.get_link_url("webui_self"))
+        with (
+            mock.patch(
+                "debusine.client.commands.work_requests.Table",
+                return_value=(table := mock.MagicMock()),
+            ),
+            mock.patch(
+                "debusine.client.commands.work_requests.rich.print"
+            ) as rich_print,
+            mock.patch("rich.json.JSON.from_data", side_effect=lambda x: x),
+        ):
+            stderr, stdout = self.capture_output(
+                partial(command._show_rich, wr)
+            )
+
+        self.assertEqual(stderr, "")
+        self.assertEqual(stdout, "")
+        rich_print.assert_called_once()
+
+        self.assertEqual(table.add_row.call_count, 6)
+        rows = [
+            call_args_list.args
+            for call_args_list in table.add_row.call_args_list
+        ]
+        self.assertEqual(
+            rows[0], ("ID:", f"#{wr.id} [link={webui_url}]{webui_url}[/]")
+        )
+        self.assertEqual(rows[1], ("Task:", f"{wr.task_type} {wr.task_name}"))
+        self.assertEqual(rows[2], ("Workspace:", f"{wr.scope}/{wr.workspace}"))
+        self.assertEqual(
+            rows[3], ("Created at:", command._format_timestamp(wr.created_at))
+        )
+        self.assertEqual(rows[4], ("Status:", wr.status))
+        self.assertEqual(
+            rows[5],
+            ("Task data:", wr.task_data),
+        )
+
+    def test_show_rich_full_data(self) -> None:
+        wr = create_work_request_response()
+        wr.started_at = wr.created_at + dt.timedelta(minutes=2)
+        wr.completed_at = wr.started_at + dt.timedelta(minutes=3)
+        wr.duration = 5
+        wr.result = WorkRequestResults.SUCCESS
+        wr.worker = 42
+        wr.priority_base = 2
+        wr.priority_adjustment = 3
+        wr.dynamic_task_data = {"dynamic": True}
+        wr.artifacts = [1, 2, 3]
+        command = self._command()
+        webui_url = command.debusine.webui_url(wr.get_link_url("webui_self"))
+        with (
+            mock.patch(
+                "debusine.client.commands.work_requests.Table",
+                return_value=(table := mock.MagicMock()),
+            ),
+            mock.patch(
+                "debusine.client.commands.work_requests.rich.print"
+            ) as rich_print,
+            mock.patch("rich.json.JSON.from_data", side_effect=lambda x: x),
+        ):
+            stderr, stdout = self.capture_output(
+                partial(command._show_rich, wr)
+            )
+
+        self.assertEqual(stderr, "")
+        self.assertEqual(stdout, "")
+        rich_print.assert_called_once()
+
+        self.assertEqual(table.add_row.call_count, 13)
+        rows = [
+            call_args_list.args
+            for call_args_list in table.add_row.call_args_list
+        ]
+        self.assertEqual(
+            rows[0], ("ID:", f"#{wr.id} [link={webui_url}]{webui_url}[/]")
+        )
+        self.assertEqual(rows[1], ("Task:", f"{wr.task_type} {wr.task_name}"))
+        self.assertEqual(rows[2], ("Workspace:", f"{wr.scope}/{wr.workspace}"))
+        self.assertEqual(
+            rows[3], ("Created at:", command._format_timestamp(wr.created_at))
+        )
+        self.assertEqual(
+            rows[4], ("Started at:", command._format_timestamp(wr.started_at))
+        )
+        self.assertEqual(
+            rows[5],
+            ("Completed at:", command._format_timestamp(wr.completed_at)),
+        )
+        self.assertEqual(
+            rows[6],
+            ("Duration:", command._format_float(wr.duration)),
+        )
+        self.assertEqual(rows[7], ("Status:", f"{wr.status} {wr.result}"))
+        self.assertEqual(
+            rows[8],
+            ("Worker:", command._format_int(wr.worker)),
+        )
+        self.assertEqual(rows[9], ("Priority:", "2 + 3"))
+        self.assertEqual(
+            rows[10],
+            ("Task data:", wr.task_data),
+        )
+        self.assertEqual(
+            rows[11],
+            ("Dynamic task data:", wr.dynamic_task_data),
+        )
+        self.assertEqual(
+            rows[12],
+            ("Artifacts:", wr.artifacts),
+        )
+
+
 class CliCreateWorkRequestTests(BaseCliTests):
     """Tests for the Cli functionality related to create work requests."""
 
@@ -72,7 +299,8 @@ class CliCreateWorkRequestTests(BaseCliTests):
 
         self.enterContext(
             self.patch_sys_stdin_read(
-                'distribution: jessie\npackage: http://..../package_1.2-3.dsc\n'
+                "distribution: stretch\n"
+                "package: http://..../package_1.2-3.dsc\n"
             )
         )
 
@@ -86,38 +314,30 @@ class CliCreateWorkRequestTests(BaseCliTests):
         cli: Cli,
         mocked_post_work_request: MagicMock,
         work_request_request: WorkRequestRequest,
-        data_from_stdin: bool = False,
     ) -> None:
         """
         Call cli.execute() and assert the work request is created.
 
         Assert expected stderr/stdout and network call.
-
-        :param data_from_stdin: if True, input data came from stdin.
         """
-        stderr, stdout = self.capture_output(cli.execute)
+        with (
+            mock.patch(
+                "debusine.client.commands.base.Command.feedback"
+            ) as feedback,
+            mock.patch(
+                "debusine.client.commands.work_requests"
+                ".WorkRequestModelCommand.show"
+            ) as show,
+        ):
+            stderr, stdout = self.capture_output(cli.execute)
 
-        base_url = self.get_base_url(self.default_server)
-        scope = self.servers[self.default_server]["scope"]
-        workspace = work_request_request.workspace or "System"
         work_request_id = mocked_post_work_request.return_value.id
-        work_request_url = urljoin(
-            base_url,
-            f"{quote(scope)}/{quote(workspace)}/"
-            f"work-request/{work_request_id}/",
-        )
 
-        expected = yaml.safe_dump(
-            {
-                'result': 'success',
-                'message': f'Work request registered: {work_request_url}',
-                'work_request_id': work_request_id,
-            },
-            sort_keys=False,
-        )
-        if data_from_stdin:
-            expected = f"---\n{expected}"
-        self.assertEqual(stdout, expected)
+        feedback.assert_called_once_with("Work request started:")
+        show.assert_called_once()
+        self.assertEqual(show.call_args.args[0].id, work_request_id)
+
+        self.assertEqual(stdout, "")
         self.assertEqual(stderr, "")
 
         actual_work_request_request = mocked_post_work_request.mock_calls[0][1][
@@ -148,7 +368,7 @@ class CliCreateWorkRequestTests(BaseCliTests):
         mocked_post_work_request = self.patch_work_request_create()
         work_request_id = 11
         data = {
-            "distribution": "jessie",
+            "distribution": "stretch",
             "package": "http://..../package_1.2-3.dsc",
         }
         mocked_post_work_request.return_value = create_work_request_response(
@@ -189,7 +409,6 @@ class CliCreateWorkRequestTests(BaseCliTests):
             cli,
             mocked_post_work_request,
             expected_work_request_request,
-            data_from_stdin=use_stdin,
         )
 
     def test_create_work_request_specific_workspace(self) -> None:
@@ -249,7 +468,7 @@ class CliCreateWorkRequestTests(BaseCliTests):
             task_data = work_request["task_data"]
             with self.subTest(task_data), self.patch_sys_stdin_read(task_data):
                 cli = self.create_cli(["work-request", "create", "task-name"])
-                stderr, stdout = self.capture_output(
+                stderr, _ = self.capture_output(
                     cli.execute, assert_system_exit_code=3
                 )
 
@@ -329,68 +548,41 @@ class CliWorkRequestListTests(BaseCliTests):
 
     def setUp(self) -> None:
         """Set up test."""
-        self.artifact = create_artifact_response(
-            id=1,
-        )
+        self.artifact = create_artifact_response(id=1)
         super().setUp()
 
     def test_list_work_requests_success(self) -> None:
         """CLI lists work requests and prints them."""
-        work_request_response_1 = create_work_request_response(id=1)
-        work_request_response_2 = create_work_request_response(id=2)
+        responses = [
+            create_work_request_response(id=1),
+            create_work_request_response(id=2),
+        ]
 
         def work_request_iter(
-            self: Debusine,
+            _self: Debusine,
         ) -> Generator[WorkRequestResponse]:
-            yield work_request_response_1
-            yield work_request_response_2
+            yield from responses
 
-        patcher = mock.patch.object(
-            Debusine, "work_request_iter", autospec=True
+        self.enterContext(
+            mock.patch.object(
+                Debusine,
+                "work_request_iter",
+                autospec=True,
+                side_effect=work_request_iter,
+            )
         )
-        mocked_work_request_iter = patcher.start()
-        mocked_work_request_iter.side_effect = work_request_iter
-        self.addCleanup(patcher.stop)
 
         cli = self.create_cli(["work-request", "list"])
 
-        stderr, stdout = self.capture_output(cli.execute)
+        with mock.patch(
+            "debusine.client.commands.work_requests.List.list"
+        ) as list_:
+            stderr, stdout = self.capture_output(cli.execute)
 
-        output = [
-            WorkRequestResponse(**kw)
-            for kw in yaml.safe_load(stdout)["results"]
-        ]
-        expected = [
-            WorkRequestResponse(
-                id=work_request_response.id,
-                url=(
-                    f"https://example.com/debusine/"
-                    f"{quote(work_request_response.workspace)}/"
-                    f"work-request/{work_request_response.id}/"
-                ),
-                created_at=work_request_response.created_at,
-                started_at=work_request_response.started_at,
-                completed_at=work_request_response.completed_at,
-                duration=work_request_response.duration,
-                status=work_request_response.status,
-                result=work_request_response.result,
-                worker=work_request_response.worker,
-                task_type=work_request_response.task_type,
-                task_name=work_request_response.task_name,
-                task_data=work_request_response.task_data,
-                dynamic_task_data=None,
-                priority_base=0,
-                priority_adjustment=0,
-                artifacts=[],
-                scope=work_request_response.scope,
-                workspace=work_request_response.workspace,
-            )
-            for work_request_response in (
-                work_request_response_1,
-                work_request_response_2,
-            )
-        ]
-        self.assertEqual(output, expected)
+        list_.assert_called_with(responses)
+
+        self.assertEqual(stdout, "")
+        self.assertEqual(stderr, "")
 
 
 class CliWorkRequestStatusTests(BaseCliTests):
@@ -404,7 +596,9 @@ class CliWorkRequestStatusTests(BaseCliTests):
         super().setUp()
 
     def artifact_get(
-        self, debusine: None, artifact_id: int  # noqa: U100
+        self,
+        debusine: None,  # noqa: ARG002, U100
+        artifact_id: int,
     ) -> ArtifactResponse:
         """
         Return copy of self.artifact changing its id to artifact_id.
@@ -413,17 +607,16 @@ class CliWorkRequestStatusTests(BaseCliTests):
           as Debusine.artifact_get() (so it can be used in a mock)
         :param artifact_id: if of the returned artifact.
         """
-        artifact = self.artifact.copy()
+        artifact = self.artifact.model_copy()
         artifact.id = artifact_id
         return artifact
 
     def test_show_work_request_success(self) -> None:
         """Cli use Debusine to fetch a WorkRequest and prints it to stdout."""
-        patcher = mock.patch.object(Debusine, 'work_request_get', autospec=True)
-        created_at = dt.datetime.now()
-        mocked_task_status = patcher.start()
+        created_at = dt.datetime.now(tz=dt.UTC)
         work_request_id = 10
         artifact_ids = [1, 2]
+
         work_request_response = create_work_request_response(
             id=work_request_id,
             created_at=created_at,
@@ -431,59 +624,39 @@ class CliWorkRequestStatusTests(BaseCliTests):
             task_data={"architecture": "amd64"},
             status="running",
         )
-        mocked_task_status.return_value = work_request_response
-        self.addCleanup(patcher.stop)
 
-        # Patch artifact_get
-        artifact_get_patcher = mock.patch.object(
-            Debusine, "artifact_get", autospec=True
+        self.enterContext(
+            mock.patch.object(
+                Debusine,
+                'work_request_get',
+                autospec=True,
+                return_value=work_request_response,
+            )
+        )
+        self.enterContext(
+            mock.patch.object(
+                Debusine,
+                "artifact_get",
+                autospec=True,
+                side_effect=self.artifact_get,
+            )
         )
-        artifact_get_mocked = artifact_get_patcher.start()
-        artifact_get_mocked.side_effect = self.artifact_get
-        self.addCleanup(artifact_get_patcher.stop)
 
         cli = self.create_cli(["work-request", "show", str(work_request_id)])
-
-        stderr, stdout = self.capture_output(cli.execute)
+        _, stdout = self.capture_output(cli.execute)
 
         expected_artifacts = []
 
         for artifact_id in artifact_ids:
             expected_artifacts.append(
-                model_to_json_serializable_dict(
-                    self.artifact_get(None, artifact_id)
-                )
+                self.artifact_get(None, artifact_id).model_dump(mode="json")
             )
 
         stdout_parsed = yaml.safe_load(stdout)
         stdout_artifacts = stdout_parsed.pop("artifacts", None)
         stdout_parsed["artifacts"] = [a["id"] for a in stdout_artifacts]
         output = WorkRequestResponse(**stdout_parsed)
-        expected = WorkRequestResponse(
-            id=work_request_response.id,
-            url=(
-                f"https://example.com/debusine/"
-                f"{quote(work_request_response.workspace)}/"
-                f"work-request/{work_request_response.id}/"
-            ),
-            created_at=work_request_response.created_at,
-            started_at=work_request_response.started_at,
-            completed_at=work_request_response.completed_at,
-            duration=work_request_response.duration,
-            status=work_request_response.status,
-            result=work_request_response.result,
-            worker=work_request_response.worker,
-            task_type=work_request_response.task_type,
-            task_name=work_request_response.task_name,
-            task_data=work_request_response.task_data,
-            dynamic_task_data=None,
-            priority_base=work_request_response.priority_base,
-            priority_adjustment=(work_request_response.priority_adjustment),
-            artifacts=[a["id"] for a in expected_artifacts],
-            scope=work_request_response.scope,
-            workspace=work_request_response.workspace,
-        )
-        self.assertEqual(output, expected)
+        self.assertEqual(output, work_request_response)
         self.assertEqual(stdout_artifacts, expected_artifacts)
 
 
@@ -590,8 +763,8 @@ class CliProvideSignatureTests(BaseCliTests):
                     + "\n".join(
                         [
                             (
-                                fr"Artifact file downloaded: "
-                                fr".*/{re.escape(path.name)}"
+                                rf"Artifact file downloaded: "
+                                rf".*/{re.escape(path.name)}"
                             )
                             for path in (dsc, buildinfo, changes)
                         ]
@@ -655,9 +828,7 @@ class CliProvideSignatureTests(BaseCliTests):
         )
         cli = self.create_cli(["work-request", "sign", "1"])
 
-        stderr, stdout = self.capture_output(
-            cli.execute, assert_system_exit_code=3
-        )
+        stderr, _ = self.capture_output(cli.execute, assert_system_exit_code=3)
 
         self.assertEqual(
             stderr,
@@ -729,7 +900,7 @@ class CliProvideSignatureTests(BaseCliTests):
             ),
             mock.patch("subprocess.run"),
         ):
-            stderr, stdout = self.capture_output(
+            stderr, _ = self.capture_output(
                 cli.execute, assert_system_exit_code=3
             )
 
@@ -918,14 +1089,10 @@ class CliOnWorkRequestTests(BaseCliTests):
             ]
         )
 
-        stderr, stdout = self.capture_output(
-            cli.execute,
-            assert_system_exit_code=3,
-        )
+        stderr, _ = self.capture_output(cli.execute, assert_system_exit_code=3)
         self.assertEqual(
             stderr,
-            'Error: write or read access '
-            f'denied for "{str(cannot_write)}"\n',
+            f'Error: write or read access denied for "{str(cannot_write)}"\n',
         )
 
     def test_debusine_on_work_request_last_completed_no_read_permission(
@@ -944,14 +1111,10 @@ class CliOnWorkRequestTests(BaseCliTests):
             ]
         )
 
-        stderr, stdout = self.capture_output(
-            cli.execute,
-            assert_system_exit_code=3,
-        )
+        stderr, _ = self.capture_output(cli.execute, assert_system_exit_code=3)
         self.assertEqual(
             stderr,
-            'Error: write or read access '
-            f'denied for "{str(cannot_write)}"\n',
+            f'Error: write or read access denied for "{str(cannot_write)}"\n',
         )
 
     def test_debusine_on_work_request_dir_last_completed_no_permission(
@@ -970,10 +1133,7 @@ class CliOnWorkRequestTests(BaseCliTests):
             ]
         )
 
-        stderr, stdout = self.capture_output(
-            cli.execute,
-            assert_system_exit_code=3,
-        )
+        stderr, _ = self.capture_output(cli.execute, assert_system_exit_code=3)
         self.assertEqual(
             stderr,
             f'Error: write access denied for directory "{str(directory)}"\n',
