@@ -16,6 +16,7 @@ from typing import Any, ClassVar, TypeAlias
 from django.test import override_settings
 from django.utils import timezone
 
+import debusine.worker.tags as wtags
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -36,7 +37,6 @@ from debusine.db.models import (
     WorkRequest,
 )
 from debusine.db.models.work_requests import SkipWorkRequest
-from debusine.server.collections.lookup import lookup_single
 from debusine.server.scheduler import schedule
 from debusine.server.workflows import (
     AutopkgtestWorkflow,
@@ -73,7 +73,7 @@ from debusine.tasks.models import (
     SbuildData,
     SbuildInput,
 )
-from debusine.test.test_utils import preserve_task_registry
+from debusine.test.django import preserve_db_task_registry
 
 
 class AutopkgtestWorkflowTests(
@@ -545,7 +545,7 @@ class AutopkgtestWorkflowTests(
             self.get_workflow(wr)._has_current_reference_qa_result("amd64")
         )
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate(self) -> None:
         """The workflow populates child work requests."""
         architectures = ("amd64", "i386")
@@ -654,7 +654,7 @@ class AutopkgtestWorkflowTests(
         children = list(WorkRequest.objects.filter(parent=autopkgtest))
         self.assertEqual(len(children), len(architectures))
 
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_populate_upload(self) -> None:
         """The workflow accepts debian:upload source artifacts."""
         architectures = ("amd64", "i386")
@@ -997,11 +997,9 @@ class AutopkgtestWorkflowTests(
             child, result=WorkRequest.Results.SUCCESS
         )
         self.assertEqual(
-            lookup_single(
+            child.lookup_single(
                 f"sid@{CollectionCategory.QA_RESULTS}/"
                 f"latest:autopkgtest_hello_amd64",
-                child.workspace,
-                user=child.created_by,
                 expect_type=LookupChildType.ARTIFACT,
             ).artifact,
             result,
@@ -1065,18 +1063,16 @@ class AutopkgtestWorkflowTests(
             ),
         )
         self.assertEqual(
-            lookup_single(
+            child.lookup_single(
                 f"sid@{CollectionCategory.QA_RESULTS}/"
                 f"latest:autopkgtest_hello_amd64",
-                child.workspace,
-                user=child.created_by,
                 expect_type=LookupChildType.ARTIFACT,
             ).collection_item,
             racing_qa_result,
         )
 
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
-    @preserve_task_registry()
+    @preserve_db_task_registry()
     def test_callback_regression_analysis(self) -> None:
         self.playground.create_collection("sid", CollectionCategory.SUITE)
         self.playground.create_collection("sid", CollectionCategory.QA_RESULTS)
@@ -1352,6 +1348,33 @@ class AutopkgtestWorkflowTests(
                 new_autopkgtest_workflow, expected_callback_statuses
             )
 
+    def test_compute_system_required_tags(self) -> None:
+        source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+        context_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="autopkgtest",
+            task_data=AutopkgtestWorkflowData(
+                source_artifact=source_artifact.id,
+                binary_artifacts=LookupMultiple((binary_artifact.id,)),
+                vendor="debian",
+                codename="trixie",
+                context_artifacts=LookupMultiple((context_artifact.id,)),
+            ),
+        )
+        workflow = self.get_workflow(wr)
+        self.assertCountEqual(
+            workflow.compute_system_required_tags(),
+            [wtags.WORKER_TYPE_NOT_ASSIGNABLE],
+        )
+
     def test_compute_dynamic_data(self) -> None:
         source_artifact = self.playground.create_source_artifact(
             name="hello", version="1.0-1"
