@@ -8,6 +8,7 @@
 # contained in the LICENSE file.
 
 """Unit tests for SystemBootstrap class."""
+
 import copy
 import hashlib
 import io
@@ -18,15 +19,11 @@ from typing import Any, cast
 from unittest import mock
 from unittest.mock import call
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
-
 import requests
 import responses
 from debian.deb822 import Deb822
 
+import debusine.worker.tags as wtags
 from debusine.client.exceptions import ContentValidationError
 from debusine.tasks import TaskConfigError
 from debusine.tasks.models import (
@@ -40,7 +37,7 @@ from debusine.tasks.models import (
 from debusine.tasks.systembootstrap import SystemBootstrap
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
-    FakeTaskDatabase,
+    MockTaskDatabase,
 )
 from debusine.test import TestCase
 
@@ -99,7 +96,7 @@ class SystemBootstrapTests(
     ) -> None:
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
-        self.task_db = FakeTaskDatabase()
+        self.task_db = MockTaskDatabase()
 
     def test_configure_task(self) -> None:
         """Ensure self.SAMPLE_TASK_DATA is valid."""
@@ -145,9 +142,20 @@ class SystemBootstrapTests(
         with self.assertRaisesRegex(TaskConfigError, msg):
             self.configure_task(task_data)
 
+    def test_get_provided_worker_tags(self) -> None:
+        self.assertCountEqual(
+            self.task.get_provided_worker_tags(),
+            [
+                wtags.TASK_PREFIX + "worker:systembootstrapimpl:version:1",
+            ],
+        )
+
+    def test_compute_system_required_tags(self) -> None:
+        self.assertCountEqual(self.task.compute_system_required_tags(), [])
+
     def test_compute_dynamic_data(self) -> None:
         self.assertEqual(
-            self.task.compute_dynamic_data(self.task_db),
+            self.compute_dynamic_data(self.task),
             BaseDynamicTaskData(
                 subject="bookworm",
                 parameter_summary="bookworm (amd64-buildd)",
@@ -167,10 +175,7 @@ class SystemBootstrapTests(
         keyring: SystemBootstrapRepositoryKeyring | None = None
         if keyring_url is not None:
             keyring = SystemBootstrapRepositoryKeyring(
-                url=pydantic.parse_obj_as(
-                    pydantic.AnyUrl | pydantic.FileUrl, keyring_url
-                ),
-                sha256sum=keyring_sha256sum,
+                url=keyring_url, sha256sum=keyring_sha256sum
             )
         repository = SystemBootstrapRepository(
             types=[SystemBootstrapRepositoryType.DEB],
@@ -257,7 +262,7 @@ class SystemBootstrapTests(
         self.assertIsNotNone(repository.keyring)
         assert repository.keyring
         responses.add(
-            responses.GET, repository.keyring.url, body=keyring_contents
+            responses.GET, str(repository.keyring.url), body=keyring_contents
         )
 
         actual = self.task._deb822_source(
@@ -286,7 +291,7 @@ class SystemBootstrapTests(
         self.assertIsNotNone(repository.keyring)
         assert repository.keyring
         responses.add(
-            responses.GET, repository.keyring.url, body=keyring_contents
+            responses.GET, str(repository.keyring.url), body=keyring_contents
         )
 
         actual = self.task._deb822_source(
@@ -319,7 +324,7 @@ class SystemBootstrapTests(
         assert repository.keyring
 
         responses.add(
-            responses.GET, repository.keyring.url, body=keyring_contents
+            responses.GET, str(repository.keyring.url), body=keyring_contents
         )
 
         actual_sha256 = hashlib.sha256(keyring_contents).hexdigest()
@@ -355,7 +360,7 @@ class SystemBootstrapTests(
         assert repository.keyring
 
         responses.add(
-            responses.GET, repository.keyring.url, body=keyring_contents
+            responses.GET, str(repository.keyring.url), body=keyring_contents
         )
 
         self.task._deb822_source(
@@ -409,9 +414,7 @@ class SystemBootstrapTests(
             check_signature_with=(
                 SystemBootstrapRepositoryCheckSignatureWith.EXTERNAL
             ),
-            keyring=SystemBootstrapRepositoryKeyring(
-                url=pydantic.parse_obj_as(pydantic.AnyUrl, keyring_url)
-            ),
+            keyring=SystemBootstrapRepositoryKeyring(url=keyring_url),
         )
 
         for use_signed_by in [True, False]:
@@ -785,15 +788,17 @@ class SystemBootstrapTests(
         assert self.task._chroot_sources_file
         assert self.task._host_sources_file
         # Source file Signed-By changed to the correct chroot path
-        with self.task._chroot_sources_file.open() as chroot_sources:
-            with self.task._host_sources_file.open() as host_sources:
-                count = 0
-                for repository in [chroot_sources, host_sources]:
-                    self.assertNotIn("Signed-By", repository)
-                    count += 1
+        with (
+            self.task._chroot_sources_file.open() as chroot_sources,
+            self.task._host_sources_file.open() as host_sources,
+        ):
+            count = 0
+            for repository in [chroot_sources, host_sources]:
+                self.assertNotIn("Signed-By", repository)
+                count += 1
 
-                # Expect two: one from chroot, one from host sources
-                self.assertEqual(count, 2)
+            # Expect two: one from chroot, one from host sources
+            self.assertEqual(count, 2)
 
         self.assertEqual(len(self.task._keyrings), 1)
         self.assertEqual(len(self.task._upload_keyrings), 0)
